

; ************************************************************
;
;					 	Displaytest_1.asm
;
;			 Test program for AVRee - developmentboard
;
;				   	      Program shows
; 
;						"   I'm AVRee    " 
;						" by A.S. & R.B. "
;
;    			       on the LCD.
;
; ************************************************************


; Include definition of registers (change to correct processor type 
; and software installation on your PC!)
; i.e.: .include "C:\Program Files\AVR Studio\AVRAssembler\Appnotes\2313def.inc"
; if 2313def.inc does not reside in the same directory as this .ASM file

.include "2313def.inc"

; Define register names
; NOTE: "Immediate"-instructions only apply to r16 - r31 !
.def	temp	= r16		; temp. storage register
.def	stack	= r17		; stack register
.def	data	= r18		; transmit data register
.def	txt 	= r19		; text output register 
.def	counter1 = r20		; Counter1 (delay) register
.def	counter2 = r21		; Counter2 (delay) register
.def	counter3 = r22		; Counter3 (delay) register
.def	status	= r0		; STATUS Register


; ************************ Source Code ************************

; *** Interrupt-Table ***

	rjmp	RESET		; Reset vector (jump address after power up)
	


textl1:		.db " SEMI CONDUCTOR ",0		; storage for first line of text
textl2:		.db "AVR  DEVELOPMENT",0		; 2nd line


; *** Program start ***
RESET:
init:
	ser	temp		; temp <- 0xFF (temp 1111 1111)
 	out	DDRB,temp	; Port B -> All pins Output
	out DDRD,temp	; Port D -> All pins Output
	cbi DDRD,0		; Port D,Bit0 Input (key S7)
	cbi DDRD,1		; Port D,Bit1 Input (key S6)
	clr	temp		; temp <- 0x00 (temp 0000 0000)
	out	PORTB,temp	; All PORTB pins -> All pins Low
	out PORTD,temp	; All PORTD pins -> All pins Low	

	clr counter1	; reset counter1
	clr counter2	; reset counter2
	clr counter3	; reset counter3
	clr stack	

	rjmp	main	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                DisplayTech 162      			;;
;;                                    			;;
;;                                    			;;
;;				  8Bit data    			;;
;;												;;
;; 				DB0-DB7:       PB0-PB7			;;
;; 				RS:            PD3    			;;
;; 				E:             PD2    			;;
;;												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

 
;---- Enable command transfer ----
data2lcd:
           out PORTB, data
  	     sbi PORTD, 2                 ;Enable high
           nop                          ;0,25 mysek delay
           nop				    ;0,25 mysek delay
           nop				    ;0,25 mysek delay
	     nop							;0,25 mysek delay
	     cbi PORTD, 2                 ;Enable low
	     out PORTB, temp		    ;clear PORTB
	     rcall delay100us		    ;call delay routine
	     ret                          

;---- Enable data transfer ----
txt2lcd:
           out PORTB, txt
	     sbi PORTD, 3			    ;RS	high
	     sbi PORTD, 2                 ;Enable high
           nop                          ;0,25 mysek delay
           nop				    ;0,25 mysek delay
           nop				    ;0,25 mysek delay
	     nop				    ;0,25 mysek delay
	     cbi PORTD, 2                 ;Enable low
	     cbi PORTD, 3			    ;RS low
	     out PORTB, temp		    ;clear PORTB
	     rcall delay100us		    ;call delay routine
	     ret                          


;---- delay after transfer 100 mysek --------------------
delay100us:                             ;0,1ms delay
           ldi  counter1, 100		    ;
delay100us_: nop				    ;0,25 mysek delay
	     dec  counter1	          ;0,25 mysek delay
           brne delay100us_		    ;0,50 mysek delay = 1mysek * 100 = 0,1ms
           ret                          

;---- Longer delay, some instructions need 5 ms ----------------------
delay5ms:                               ;5ms delay
           ldi  counter2, 50		    ;50 * 0,1ms = 5ms

delay1:                                 ;0,1ms delay
           ldi  counter1, 100			;
delay1_:   nop	                      ;0,25 mysek delay
	     dec  counter1    		    ;0,25 mysek delay
           brne delay1_  		    ;0,50 mysek delay = 1mysek * 100 = 0,1ms

           dec  counter2				
           brne delay1					
           ret                          

;Init DISPLAY at start of program
init_lcd:
           ldi	counter3,3	          ;3* 5ms = 15ms
powerupwait:				    ;15ms delay for display POWER-UP
           rcall	delay5ms		
           dec	counter3		    ;
           brne	powerupwait		    ;next 5ms delay
           ldi data, 0b00110000         ;must be sent three times
           out PORTB, data              ;for display initialisation
           rcall data2lcd               ;1
	     rcall delay5ms		    ;Wait 5ms the first time!
	     out PORTB, data 				;
	     rcall data2lcd               ;2
           out PORTB, data				;
	     rcall data2lcd               ;and 3!
           
           ldi data, 0b00111000         ;Display: 2 lines 5x8 dots(8 bit mode)
           out PORTB, data		    ;
           rcall data2lcd		    ;
           
	     ldi data, 0b00001100         ;Display on, cursor off, cursor blink off;
	     out PORTB, data			
	     rcall data2lcd				
		   
	     ret							


;Clear display
clear_lcd:
           ldi data, 0b00000001   	    ;clear display  command
           rcall data2lcd		    ;
           rcall delay5ms		    ;
	     ret				    


;text output routine with delay

charout:   ldi	counter3,50		    ;50* 5ms = 250ms
wait:	     rcall  delay5ms		    ;
           dec	counter3		    ;delay to display characters
           brne	wait			    ;one by one.

	     lpm				    ;load program memory
	     tst	status	          ;check status register 
	     breq	charend		    ;Status reg = 0x00 = end of text
	     mov	txt,status		    ;store status reg in  Register txt
	     rcall txt2lcd		    ;send character to dispaly
	     inc	ZL			    ;increment ZL for next character
	     rjmp	charout		    ;next character
charend:   ret	


;**********************Main program***********************

main:      ldi stack, RAMEND    	    ;initialise stack pointer 
           out SPL, stack

           rcall init_lcd		    ;initialise display 
	     rcall clear_lcd     	    ;clear display 


text:      ldi data, 0b00000000	    ;Cursor home instruction (1st line, 1st char. pos.)
	     rcall data2lcd		    ;send instruction
	     clr	ZH			    ;clear ZH for text output
	     ldi	ZL,low(textl1*2)	    ;load character from table textl1 in ZL
	     rcall charout		    ;send character

	     ldi data, 0b11000000	    ;Cursor to 2nd line, 1st char. position
	     rcall data2lcd		    ;send instruction
	     clr	ZH			    ;clear ZH 
	     ldi	ZL,low(textl2*2)	    ;load character from table textl2 in ZL
	     rcall charout		    ;send character

end:	     sbis PIND,PD0		    ;check Port D PD0 (key S7)
	     rjmp end					;
	     rcall clear_lcd		    ;clear display
	     rjmp text			    ;and display same text again
