

; ************************************************************
;
;				  altonoff.asm
;
;			 Test program for AVRee  development board
;
;				  alternating on/off PORTB
; key press on start up:
; S6:	on/off
; S7: alternating		    
;
; ************************************************************
;
; Include definition of registers (change to correct processor type 
; and software installation on your PC!)
; i.e.: .include "C:\Program Files\AVR Studio\AVRAssembler\Appnotes\2313def.inc"
; if 2313def.inc does not reside in the same directory as this .ASM file
.include "2313def.inc"

; Define register names
; NOTE: "Immediate"-instructions only apply to r16 - r31 !
.def	temp	= r16		; temp. storage
.def	ledsw	= r17		; LED register (alternating mode)
.def	ledsd	= r18		; LED register (on/off mode)
.def	status= r0		; STATUS-Register


; *** Interrupt-Table ***

	rjmp	RESET		; Reset vector
			

; *** Program start ***
RESET:
init:
	ser	temp		; temp <- 0xFF (temp = 1111 1111)
 	out	DDRB,temp	; Port B -> All pins output
	clr	temp		; temp <- 0x00 (temp = 0000 0000)
	out DDRD,temp	; Port D -> All pins input
	out	PORTB,temp	; All pins PORTB Low
	out PORTD,temp	; All pins PORTD Low	
	ldi	ledsw,0x0F	; (alternating mode)Register  ledsw = 1111 0000 
	ldi ledsd,0xFF	; (on/off mode) Register ledsd = 1111 1111 
	
	rjmp main		; 

;*********on/off mode***********

onoff:		out PORTB,ledsd		; Register ledsd to PORTB 
			com ledsd			; complement ledsd 
			rjmp onoff			; endless loop	
	
;*********alternating mode***********	

alter:		out	PORTB,ledsw		; Register ledsw to PORTB 
			nop
			nop
			nop
			nop
			com ledsw			; complement ledsw (0000 1111 or 1111 0000)
			rjmp alter 		      ; alternating mode (repeat forever)

;------------MAIN---------------

main: 		sbic PIND,PD0		; check PORTD pin PD0 (key S7) 	
			rcall alter 		; alternating mode
			sbic PIND,PD1		; check pin PD1 (key S6) 
	  		rcall onoff			; on/off mode
			rjmp main			;